/*************************************************************************/
/*
 *  Datei           : calctype.h
 *  Inhalt          : Deklarationen der Klassen :
 *                        o FilePosition
 *                        o VariablenInhalt
 *                        o Variablen
 *                        o CalcObjectDefLocus
 *                      Weitere Deklarationen :
 *                        o Print
 *                        o YYSTYPE
 *  Programmierer   : Sven Goethel
 *                      Stapenhorststr.35a
 *                      33615 Bielefeld
 *                      0521/139228
 *  Erstellungsdatum: 06.07.1994
 *  nderungsdatum  : 14.12.1994
 *  Projekt         : calc
 *  Verzeichnis     :
 *  Dokumentation   : calc.doc
 *
 *
 * $Log: calctype.h  $
 * Revision 2.26  1995/06/06 08:48:27  Sven_Goethel_DFP
 * ok
 * Revision 2.25  1995/06/06 08:20:25  Sven_Goethel_DFP
 * lokale char Buffer mit fester Groesse -> lokale StrClass (dynamische groesse)
 * Revision 2.24  1995/04/07 11:35:11  Sven_Goethel_DFP
 * t_string wieder zurueck zu StrClass
 * Alles angepasst fuer Horts
 * ( aehh Horst . )
 * Wenn __SHOW_CALC__ definiert ist,
 * so werden Zeichen auf stderr gedruckt bei :
 *   PARSEN der DATEI
 *   INTERPRETIEREN der DATENSTRUKTUR
 * und alles fuer Calculator NEU und OLD
 * Revision 2.22  1995/04/05 07:45:40  Sven_Goethel_DFP
 * Alle Fehler behoben ..
 * getestet mit und ohne __OLD_INTERPRETE__ !
 * Fuer den neuen Interpreter sind Fehler-Message (FILE und ZEILENNUMMER)
 * hinzugekommen.
 * Fuer __OLD_INTERPRETE__ wurde ein BUG entfernt :
 *   bei return innerhalb verschachtelter Bloecke wurde der 
 *   Block zaehler nicht wieder zureuckgesetzt !!!
 * Revision 2.21  1995/04/05 02:00:27  Sven_Goethel_DFP
 * profekt-pflege nun mit KornShell-Sripts
 * Revision 2.2  1995/04/05 01:48:19  Sven_Goethel_DFP
 * __Mem_Free fehler beseitigt
 * Das MKS System hat um ein mittels new allokiertes Array zu deallokieren,
 * den delete operator verwendet.
 * Es muss aber der delete[] operator verwendet werden :
 *   int *p=new int[100];
 *   delete p; // Error !
 *   delete[] p; // Ok !
 * Revision 2.1  1995/04/01 02:41:33  Sven_Goethel_DFP
 * Referenz fuer Globale Variablen beseitigt, da FEHLER !!!!
 * Das Watcom-Executable stuerzt ab, wenn mehr als zwei ELSE zweige
 * existieren ( oder schon ab zwei ??) <- im test mit test.clc zu ersehen.
 * Das Borland C++ 4.0 executable hingegen arbeitet fehlerfrei !!!
 * Revision 1.6  1995/02/28 15:07:46  Sven_Goethel_WPP
 * source ist nun mit borland c++ version 4.0
 * und mit watcom c++ version 10.0 zu uebersetzen !!!!
 * Revision 1.2  1995/01/30 08:17:59  Sven_Goethel
 * 1st full implemented & checked version
 * Revision 1.1  1995/01/27 17:45:06  Sven_Goethel
 * Initial revision
 * Revision 1.1  1994/12/16 16:52:07  Sven_Goethel
 * Initial revision
 *
 *    (C) Copyright Dr.-Ing.F.Prekwinkel GmbH, 32052 Herford, Germany
 */
/*************************************************************************/

# ifndef _CALCTYPE_H
  # define _CALCTYPE_H

  # include <stdio.h>
  # include <math.h>
  # include "liste.h"
  # include "strclass.h"
  # include "maschine.h"
  # include "sortlist.h"
  # include "hashlist.h"

  # ifndef NDEBUG
    # define DPRINT(a) printf((a))
  # else
    # define DPRINT(a)
  # endif
  # ifndef M_PI
    # define M_PI        3.14159265358979323846
  #endif


  // Alle ElementFunktionen haben folgende Rueckgabelogik :
  // Return 0 ==> FEHLER
  // Return 1 ==> OK

  // const int BufferSize=200 ... definiert in lex_yy.hpp !!!

  typedef enum { NONE, CHAR, STRING, REELL, INT, COORD } VarType;

  class FilePosition {
  public:
    int operator==(const FilePosition&) const ;
    long ByteNo;
    int  LineNo;
  };

  ostream& operator<< (ostream& OS, FilePosition& pos);

  class VariablenInhalt {
  public:
      VariablenInhalt(void)
      : Level(0), Type(NONE), rInhalt(0), strInhalt() {}
      VariablenInhalt(const VariablenInhalt &m)
      : Level(m.Level), Type(m.Type),
        rInhalt(m.rInhalt), strInhalt(m.strInhalt) {}
      VariablenInhalt(int Level, VarType Type, double rInhalt=0, const char *strInhalt=NULL)
      : Level(Level), Type(Type),
        rInhalt(rInhalt), strInhalt(strInhalt) {}
      virtual ~VariablenInhalt() {}
      VariablenInhalt& operator=(const VariablenInhalt&);
      int operator==(const VariablenInhalt&) const ;
      int operator>=(const VariablenInhalt&) const ;
      int operator<=(const VariablenInhalt&) const ;
      int operator>(const VariablenInhalt&) const ;
      int operator<(const VariablenInhalt&) const ;
      int operator!=(const VariablenInhalt&) const ;
      int operator&&(const VariablenInhalt&) const ;
      int operator||(const VariablenInhalt&) const ;
      VarType GetBiggestType(const VariablenInhalt &) const ;
      void Set(VarType Type=NONE, int Level=0, double rInhalt=0, char *m=NULL);
      operator int() 
      { if(Type==STRING) return (int)strInhalt;
        else return (int)rInhalt; 
      }

      VarType Type;
      int Level;
      double    rInhalt;
      StrClass strInhalt;
  };

  // Die zu definierende Hash-Funktion
  class HashStrClass : 
  public HashListe<StrClass> 
  {
  public:
    HashStrClass(int S=1) : HashListe<StrClass>(S) {}
    HashStrClass(const HashStrClass &m) : HashListe<StrClass>(m) {}
    virtual ~HashStrClass() {}
    virtual int HashFunktion(const StrClass &a) const;
  };

  ostream& operator<< (ostream& OS, HashListe<StrClass>& hsc);

  class LevelVariablen {
  public:
      LevelVariablen(int Level);
      LevelVariablen(const LevelVariablen *ptrVars);
      LevelVariablen(const LevelVariablen &Vars);
      virtual ~LevelVariablen(void) { }
      LevelVariablen &operator=(const LevelVariablen &m)
      {
        Copy(&m);
        return *this;
      }
      int operator==(const LevelVariablen &m) const
      { return Level==m.Level; }
      int operator==(int level) const
      { return Level==level; }
      int operator!=(int level) const
      { return Level!=level; }
      int HoleLevel(void) { return Level; }

      HashStrClass                      VarNamen;
      Liste<VariablenInhalt>          VarInhalt;
  private:
      int Level;
      void Copy(const LevelVariablen *Vars);
  };

  ostream& operator<< (ostream& OS, LevelVariablen& levVar);

  class Variablen {
  #ifndef __OLD_INTERPRETE__
    friend class Zuweisung;
    friend class VarOperand;
  #endif
  public:
      Variablen(void) : LevelVariablenListe() {}
      Variablen(const Variablen *ptrVars);
      Variablen(const Variablen &Vars);
      Variablen &operator=(const Variablen &Vars);
      virtual ~Variablen(void) { Dest(); }
      int HoleWert ( int Level, const char *VarName , VariablenInhalt &VarInhalt);
      int SetzeWert ( int Level, const char *VarName , const VariablenInhalt &VarInhalt);
      # ifdef __OLD_INTERPRETE__
        int SetzeWerte ( int Level, Liste<VariablenInhalt> &ArgListe);
      # endif
      int DefiniereVar ( const char *VarName , const VariablenInhalt &VarVal);
      int Loesche ( int Level );
      void Dest(void);
      # ifdef __OLD_INTERPRETE__
        void ClearVarDefOrder();
      # endif
  private:
      VariablenInhalt *DeRefVar ( int LevelVon, int LevelBis, const char *VarName );
      Liste<LevelVariablen>            LevelVariablenListe;
      # ifdef __OLD_INTERPRETE__
        // Die Reihenfolge der Variablendefinitionen muss fuer die
        // Funktionsparameteruebergabe gemerkt werden,
        // da die Variablen in einer HashListe gespeichert sind !!!
        Liste<StrClass>                VarDefOrder;
      # endif
  };

  class LabelLocus {
  public:
      int GetFilePos(const StrClass &Name, FilePosition &result);
      int SetFilePos(const StrClass &Name, const FilePosition& );
      int laenge(void) ;
      int loesche(int i) ;
  private:
      Liste<StrClass>         Namen;
      Liste<FilePosition>     FilePos;
  };

  int Print(FILE *fPtr, const char *str, Liste<VariablenInhalt> &ArgListe);

# endif


